/**
 * DevExtreme (esm/ui/scheduler/resources/resourceManager.js)
 * Version: 21.2.0 (build 21243-0310)
 * Build date: Tue Aug 31 2021
 *
 * Copyright (c) 2012 - 2021 Developer Express Inc. ALL RIGHTS RESERVED
 * Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
 */
import {
    wrapToArray,
    inArray
} from "../../../core/utils/array";
import {
    grep
} from "../../../core/utils/common";
import {
    isDefined
} from "../../../core/utils/type";
import {
    deepExtendArraySafe
} from "../../../core/utils/object";
import {
    each
} from "../../../core/utils/iterator";
import {
    extend
} from "../../../core/utils/extend";
import query from "../../../data/query";
import {
    compileGetter,
    compileSetter
} from "../../../core/utils/data";
import {
    when,
    Deferred
} from "../../../core/utils/deferred";
import {
    AgendaResourceProcessor
} from "./agendaResourceProcessor";
import {
    createResourcesTree,
    getCellGroups,
    getDisplayExpr,
    getFieldExpr,
    getValueExpr,
    getWrappedDataSource
} from "./utils";
export class ResourceManager {
    constructor(resources) {
        this.loadedResources = [];
        this._resourceLoader = {};
        this._dataAccessors = {
            getter: {},
            setter: {}
        };
        this.agendaProcessor = new AgendaResourceProcessor;
        this.setResources(resources)
    }
    _isMultipleResource(resourceField) {
        var result = false;
        each(this.getResources(), function(_, resource) {
            var field = getFieldExpr(resource);
            if (field === resourceField) {
                result = resource.allowMultiple;
                return false
            }
        }.bind(this));
        return result
    }
    getDataAccessors(field, type) {
        var result = null;
        each(this._dataAccessors[type], (function(accessorName, accessors) {
            if (field === accessorName) {
                result = accessors;
                return false
            }
        }));
        return result
    }
    setResources() {
        var resources = arguments.length > 0 && void 0 !== arguments[0] ? arguments[0] : [];
        this._resources = resources;
        this._dataAccessors = {
            getter: {},
            setter: {}
        };
        this._resourceFields = resources.map(resource => {
            var field = getFieldExpr(resource);
            this._dataAccessors.getter[field] = compileGetter(field);
            this._dataAccessors.setter[field] = compileSetter(field);
            return field
        });
        this.agendaProcessor.initializeState(resources)
    }
    getResources() {
        return this._resources || []
    }
    getEditors() {
        return this.getResources().map(resource => {
            var dataField = getFieldExpr(resource);
            var dataSource = this._getResourceDataByField(dataField);
            return {
                editorOptions: {
                    dataSource: dataSource.length ? dataSource : getWrappedDataSource(resource.dataSource),
                    displayExpr: getDisplayExpr(resource),
                    valueExpr: getValueExpr(resource)
                },
                dataField: dataField,
                editorType: resource.allowMultiple ? "dxTagBox" : "dxSelectBox",
                label: {
                    text: resource.label || dataField
                }
            }
        })
    }
    getResourceDataByValue(field, value) {
        var result = new Deferred;
        this.getResources().forEach(resource => {
            var resourceField = getFieldExpr(resource);
            if (resourceField === field) {
                var dataSource = getWrappedDataSource(resource.dataSource);
                var valueExpr = getValueExpr(resource);
                if (!this._resourceLoader[field]) {
                    this._resourceLoader[field] = dataSource.load()
                }
                this._resourceLoader[field].done(data => {
                    var filteredData = query(data).filter(valueExpr, value).toArray();
                    delete this._resourceLoader[field];
                    result.resolve(filteredData[0])
                }).fail(() => {
                    delete this._resourceLoader[field];
                    result.reject()
                })
            }
        });
        return result.promise()
    }
    setResourcesToItem(itemData, resources) {
        var resourcesSetter = this._dataAccessors.setter;
        for (var name in resources) {
            if (Object.prototype.hasOwnProperty.call(resources, name)) {
                var resourceData = resources[name];
                resourcesSetter[name](itemData, this._isMultipleResource(name) ? wrapToArray(resourceData) : resourceData)
            }
        }
    }
    getResourcesFromItem(itemData, wrapOnlyMultipleResources) {
        var result = null;
        if (!isDefined(wrapOnlyMultipleResources)) {
            wrapOnlyMultipleResources = false
        }
        this._resourceFields.forEach(field => {
            each(itemData, (fieldName, fieldValue) => {
                var tempObject = {};
                tempObject[fieldName] = fieldValue;
                var resourceData = this.getDataAccessors(field, "getter")(tempObject);
                if (isDefined(resourceData)) {
                    if (!result) {
                        result = {}
                    }
                    if (1 === resourceData.length) {
                        resourceData = resourceData[0]
                    }
                    if (!wrapOnlyMultipleResources || wrapOnlyMultipleResources && this._isMultipleResource(field)) {
                        this.getDataAccessors(field, "setter")(tempObject, wrapToArray(resourceData))
                    } else {
                        this.getDataAccessors(field, "setter")(tempObject, resourceData)
                    }
                    extend(result, tempObject);
                    return true
                }
            })
        });
        return result
    }
    isLoaded() {
        return isDefined(this.loadedResources)
    }
    _mapResourceData(resource, data) {
        var valueGetter = compileGetter(getValueExpr(resource));
        var displayGetter = compileGetter(getDisplayExpr(resource));
        return data.map(item => {
            var result = {
                id: valueGetter(item),
                text: displayGetter(item)
            };
            if (item.color) {
                result.color = item.color
            }
            return result
        })
    }
    loadResources(groups) {
        var _this = this;
        var result = new Deferred;
        var deferreds = [];
        this.getResourcesByFields(groups).forEach(resource => {
            var deferred = new Deferred;
            var name = getFieldExpr(resource);
            deferreds.push(deferred);
            getWrappedDataSource(resource.dataSource).load().done(data => {
                var items = this._mapResourceData(resource, data);
                deferred.resolve({
                    name: name,
                    items: items,
                    data: data
                })
            }).fail(() => deferred.reject())
        });
        if (!deferreds.length) {
            this.loadedResources = [];
            return result.resolve([])
        }
        when.apply(null, deferreds).done((function() {
            for (var _len = arguments.length, resources = new Array(_len), _key = 0; _key < _len; _key++) {
                resources[_key] = arguments[_key]
            }
            var hasEmpty = resources.some(r => 0 === r.items.length);
            _this.loadedResources = hasEmpty ? [] : resources;
            result.resolve(_this.loadedResources)
        })).fail(() => result.reject());
        return result.promise()
    }
    getResourcesByFields(fields) {
        return grep(this.getResources(), function(resource) {
            var field = getFieldExpr(resource);
            return inArray(field, fields) > -1
        }.bind(this))
    }
    getResourceByField(field) {
        return this.getResourcesByFields([field])[0] || {}
    }
    getResourceColor(field, value) {
        var valueExpr = this.getResourceByField(field).valueExpr || "id";
        var valueGetter = compileGetter(valueExpr);
        var colorExpr = this.getResourceByField(field).colorExpr || "color";
        var colorGetter = compileGetter(colorExpr);
        var result = new Deferred;
        var resourceData = this._getResourceDataByField(field);
        var resourceDataLength = resourceData.length;
        var color;
        if (resourceDataLength) {
            for (var i = 0; i < resourceDataLength; i++) {
                if (valueGetter(resourceData[i]) === value) {
                    color = colorGetter(resourceData[i]);
                    break
                }
            }
            result.resolve(color)
        } else {
            this.getResourceDataByValue(field, value).done((function(resourceData) {
                if (resourceData) {
                    color = colorGetter(resourceData)
                }
                result.resolve(color)
            })).fail((function() {
                result.reject()
            }))
        }
        return result.promise()
    }
    getResourceForPainting(groups) {
        var resources = this.getResources();
        var result;
        each(resources, (function(index, resource) {
            if (resource.useColorAsDefault) {
                result = resource;
                return false
            }
        }));
        if (!result) {
            if (Array.isArray(groups) && groups.length) {
                resources = this.getResourcesByFields(groups)
            }
            result = resources[resources.length - 1]
        }
        return result
    }
    _hasGroupItem(appointmentResources, groupName, itemValue) {
        var group = this.getDataAccessors(groupName, "getter")(appointmentResources);
        if (group) {
            if (inArray(itemValue, group) > -1) {
                return true
            }
        }
        return false
    }
    _createPlainResourcesByAppointmentAsync(rawAppointment) {
        return this.agendaProcessor.createListAsync(rawAppointment)
    }
    _getResourceDataByField(fieldName) {
        for (var i = 0; i < this.loadedResources.length; i++) {
            var resource = this.loadedResources[i];
            if (resource.name === fieldName) {
                return resource.data
            }
        }
        return []
    }
    getResourceTreeLeaves(tree, appointmentResources, result) {
        result = result || [];
        for (var i = 0; i < tree.length; i++) {
            if (!this._hasGroupItem(appointmentResources, tree[i].name, tree[i].value)) {
                continue
            }
            if (isDefined(tree[i].leafIndex)) {
                result.push(tree[i].leafIndex)
            }
            if (tree[i].children) {
                this.getResourceTreeLeaves(tree[i].children, appointmentResources, result)
            }
        }
        return result
    }
    groupAppointmentsByResources(appointments, groups) {
        var result = {
            0: appointments
        };
        if (groups && groups.length && this.loadedResources.length) {
            result = this.groupAppointmentsByResourcesCore(appointments, this.loadedResources)
        }
        var totalResourceCount = 0;
        each(this.loadedResources, (function(i, resource) {
            if (!i) {
                totalResourceCount = resource.items.length
            } else {
                totalResourceCount *= resource.items.length
            }
        }));
        for (var j = 0; j < totalResourceCount; j++) {
            var index = j.toString();
            if (result[index]) {
                continue
            }
            result[index] = []
        }
        return result
    }
    groupAppointmentsByResourcesCore(appointments, resources) {
        var tree = createResourcesTree(resources);
        var result = {};
        each(appointments, function(_, appointment) {
            var appointmentResources = this.getResourcesFromItem(appointment);
            var treeLeaves = this.getResourceTreeLeaves(tree, appointmentResources);
            for (var i = 0; i < treeLeaves.length; i++) {
                if (!result[treeLeaves[i]]) {
                    result[treeLeaves[i]] = []
                }
                result[treeLeaves[i]].push(deepExtendArraySafe({}, appointment, true))
            }
        }.bind(this));
        return result
    }
    getAppointmentColor(options) {
        var {
            groups: groups
        } = options;
        var resourceForPainting = this.getResourceForPainting(groups);
        var response = (new Deferred).resolve().promise();
        if (resourceForPainting) {
            var field = getFieldExpr(resourceForPainting);
            var {
                groupIndex: groupIndex,
                itemData: itemData
            } = options;
            var cellGroups = getCellGroups(groupIndex, this.loadedResources);
            var resourceValues = wrapToArray(this.getDataAccessors(field, "getter")(itemData));
            var groupId = resourceValues.length ? resourceValues[0] : void 0;
            for (var i = 0; i < cellGroups.length; i++) {
                if (cellGroups[i].name === field) {
                    groupId = cellGroups[i].id;
                    break
                }
            }
            response = this.getResourceColor(field, groupId)
        }
        return response
    }
    reduceResourcesTree(tree, existingAppointments, _result) {
        _result = _result ? _result.children : [];
        var that = this;
        tree.forEach((function(node, index) {
            var ok = false;
            var resourceName = node.name;
            var resourceValue = node.value;
            var resourceTitle = node.title;
            var resourceData = node.data;
            var resourceGetter = that.getDataAccessors(resourceName, "getter");
            existingAppointments.forEach((function(appointment) {
                if (!ok) {
                    var resourceFromAppointment = resourceGetter(appointment);
                    if (Array.isArray(resourceFromAppointment)) {
                        if (resourceFromAppointment.indexOf(resourceValue) > -1) {
                            _result.push({
                                name: resourceName,
                                value: resourceValue,
                                title: resourceTitle,
                                data: resourceData,
                                children: []
                            });
                            ok = true
                        }
                    } else if (resourceFromAppointment === resourceValue) {
                        _result.push({
                            name: resourceName,
                            value: resourceValue,
                            title: resourceTitle,
                            data: resourceData,
                            children: []
                        });
                        ok = true
                    }
                }
            }));
            if (ok && node.children && node.children.length) {
                that.reduceResourcesTree(node.children, existingAppointments, _result[index])
            }
        }));
        return _result
    }
    getResourcesDataByGroups(groups) {
        if (!groups || !groups.length) {
            return this.loadedResources
        }
        var fieldNames = {};
        var currentResourcesData = [];
        groups.forEach(group => {
            each(group, (name, value) => fieldNames[name] = value)
        });
        var resourceData = this.loadedResources.filter(_ref => {
            var {
                name: name
            } = _ref;
            return isDefined(fieldNames[name])
        });
        resourceData.forEach(data => currentResourcesData.push(extend({}, data)));
        currentResourcesData.forEach(currentResource => {
            var {
                items: items,
                data: data,
                name: resourceName
            } = currentResource;
            var resource = this.getResourceByField(resourceName);
            var valueExpr = getValueExpr(resource);
            var filteredItems = [];
            var filteredData = [];
            groups.filter(group => isDefined(group[resourceName])).forEach(group => {
                each(group, (name, value) => {
                    if (!filteredItems.filter(item => item.id === value && item[valueExpr] === name).length) {
                        var currentItems = items.filter(item => item.id === value);
                        var currentData = data.filter(item => item[valueExpr] === value);
                        filteredItems.push(...currentItems);
                        filteredData.push(...currentData)
                    }
                })
            });
            currentResource.items = filteredItems;
            currentResource.data = filteredData
        });
        return currentResourcesData
    }
    createReducedResourcesTree(appointments) {
        var tree = createResourcesTree(this.loadedResources);
        return this.reduceResourcesTree(tree, appointments)
    }
}
